/****************************************************************************
 * (c) 2019 softwareentwicklung-als-prozess.de
 * Funktionalitaet:
 * Ausgabe auf zweizeiligem Display
 ***************************************************************************/
#include <arduino.h>
#include "i2clcd.h"
#include "lcd.h"

// eigene defines
#define DISPLAY_LENGTH 16
#define SEKUNDE 1000
#define BEWEGUNG 100
#define MOVE 333

// interne Variable
static char    lcdtxt[DISPLAY_LENGTH + 1];
static char   lcdoben[DISPLAY_LENGTH + 1];
static char  lcdunten[DISPLAY_LENGTH + 1];
static char  lcdoben2[DISPLAY_LENGTH + 1];
static char lcdunten2[DISPLAY_LENGTH + 1];
static char shftlnksmrkr   = (char)BLANK;
static char shftrchtsmrkr  = (char)BLANK;
static char shftlnksmrkr2  = (char)BLANK;
static char shftrchtsmrkr2 = (char)BLANK;

// interne Prototypen
static void LCD_Intro(void);
static void LCD_Shift_Links(char* txt);
static void LCD_Shift_Rechts(char* txt);
static void LCD_Shift_Links2(char* txt);
static void LCD_Shift_Rechts2(char* txt);

static void LCD_Show_Single(void);
static void LCD_Show_Two(void);

static void LCD_Augen_Blick(void);
static void LCD_Augen_Blinzeln(void);

void lcd_Setup(void)
{
  uint8_t i = (uint8_t)0;

  for (i = 0; i < (DISPLAY_LENGTH); i++)
  {
    lcdtxt[i] = BLANK;
  }
  lcdtxt[DISPLAY_LENGTH] = ENDE;
}

void lcd_AugenDemo(void) 
{
  uint8_t i     = (uint8_t)0;
  uint8_t j     = (uint8_t)0;
  shftlnksmrkr  = (char)BLANK;
  shftrchtsmrkr = (char)BLANK;
  
  LCD_Intro();
  delay(SEKUNDE);
  i2clcd_Clear_All();
  lcdtxt[6] = lcdtxt[7] = lcdtxt[8] = lcdtxt[9] = (uint8_t)1; // Pupille in die Mitte setzen
  LCD_Augen_Blick();
  delay(SEKUNDE);
  for (i = (uint8_t)0; i < (uint8_t)6; i++) // von der Mitte nach linken Anschlag
  {
    LCD_Shift_Links(lcdtxt);
    LCD_Augen_Blick();
    delay(BEWEGUNG);
  }
  delay(SEKUNDE);
  while(1)
  {
    for (i = (uint8_t)0; i < (uint8_t)12; i++) // von linken Anschlag nach rechten Anschlag
    {
      LCD_Shift_Rechts(lcdtxt);
      LCD_Augen_Blick();
      delay(BEWEGUNG);
    }
    delay(SEKUNDE);
    for (i = (uint8_t)0; i < (uint8_t)12; i++) // von linken Anschlag nach rechten Anschlag
    {
      LCD_Shift_Links(lcdtxt);
      LCD_Augen_Blick();
      delay(BEWEGUNG);
    }
    delay(SEKUNDE);
    j++;
    if ( (uint8_t)10 == j)
    {
      LCD_Augen_Blinzeln();
      j = (uint8_t)0;
    }
  }
}

void lcd_RundInit(void)
{
  strcpy(lcdoben,"ABCDEFGHIJKLMNOP");
  strcpy(lcdunten,"                ");  
  strcpy(lcdoben2,"QRSTUVWXYZ123456");
  strcpy(lcdunten2,"                ");  
}

void lcd_Two_RundRechts(void) // rechts herum auf zwei LCD's
{
  uint8_t i = (uint8_t)0;
  
  LCD_Show_Two();
  for(i = (uint8_t)0; i < DISPLAY_LENGTH; i++)
  {
    shftrchtsmrkr  = lcdunten[0];
    shftrchtsmrkr2 = lcdoben[15];
    shftlnksmrkr2  = lcdoben2[15];
    shftlnksmrkr   = lcdunten2[0];
    LCD_Shift_Rechts2(lcdoben2);
    LCD_Shift_Rechts(lcdoben);
    LCD_Shift_Links2(lcdunten2);
    LCD_Shift_Links(lcdunten);
    delay(MOVE);
    LCD_Show_Two();
  }
}

void lcd_Two_RundLinks(void) // links herum auf zwei LCD's
{
  uint8_t i = (uint8_t)0;
  
  LCD_Show_Two();
  for(i = (uint8_t)0; i < DISPLAY_LENGTH; i++)
  {
    shftrchtsmrkr  = lcdoben[0];
    shftrchtsmrkr2 = lcdunten[15];
    shftlnksmrkr2  = lcdunten2[15];
    shftlnksmrkr   = lcdoben2[0];
    LCD_Shift_Rechts2(lcdunten2);
    LCD_Shift_Rechts(lcdunten);
    LCD_Shift_Links2(lcdoben2);
    LCD_Shift_Links(lcdoben);
    delay(MOVE);
    LCD_Show_Two();
  }
}

void lcd_Sngl_RundRechts(void) // rechts herum auf einem LCD
{
  uint8_t i = (uint8_t)0;
  
  LCD_Show_Single();
  for(i = (uint8_t)0; i < DISPLAY_LENGTH; i++)
  {
    shftrchtsmrkr = lcdunten[0];
    shftlnksmrkr  = lcdoben[15];
    LCD_Shift_Rechts(lcdoben);
    LCD_Shift_Links(lcdunten);
    delay(MOVE);
    LCD_Show_Single();
  }
}

void lcd_Sngl_RundLinks(void) // links herum auf einem Display
{  uint8_t i = (uint8_t)0;
  
  LCD_Show_Single();
  for(i = (uint8_t)0; i < DISPLAY_LENGTH; i++)
  {
    shftrchtsmrkr = lcdoben[0];
    shftlnksmrkr  = lcdunten[15];
    LCD_Shift_Rechts(lcdunten);
    LCD_Shift_Links(lcdoben);
    delay(MOVE);
    LCD_Show_Single();
  }
}

static void LCD_Intro(void)
{
  i2clcd_Write_Links(I2CLCD_LINE1, " linkes Display ");
  i2clcd_Write_Rechts(I2CLCD_LINE1, " rechtes Display");  
}

static void LCD_Shift_Links(char * txt)
{
  uint8_t i = (uint8_t)0;

  for (i = (uint8_t)1; i < (uint8_t)DISPLAY_LENGTH; i++)
  {
    txt[i - 1] = txt[i];
  }
  txt[DISPLAY_LENGTH - 1] = shftlnksmrkr;
}

static void LCD_Shift_Rechts(char* txt)
{
  uint8_t i = (uint8_t)0;

  for (i = (uint8_t)(DISPLAY_LENGTH - 1); i > (uint8_t)0; i--)
  {
    txt[i] = txt[i - 1];
  }
  txt[0] = shftrchtsmrkr;
}

static void LCD_Shift_Links2(char * txt)
{
  uint8_t i = (uint8_t)0;

  for (i = (uint8_t)1; i < (uint8_t)DISPLAY_LENGTH; i++)
  {
    txt[i - 1] = txt[i];
  }
  txt[DISPLAY_LENGTH - 1] = shftlnksmrkr2;
}

static void LCD_Shift_Rechts2(char* txt)
{
  uint8_t i = (uint8_t)0;

  for (i = (uint8_t)(DISPLAY_LENGTH - 1); i > (uint8_t)0; i--)
  {
    txt[i] = txt[i - 1];
  }
  txt[0] = shftrchtsmrkr2;
}

static void LCD_Show_Two(void)
{
  i2clcd_Write_Links(I2CLCD_LINE1, lcdoben);
  i2clcd_Write_Links(I2CLCD_LINE2, lcdunten);
  i2clcd_Write_Rechts(I2CLCD_LINE1, lcdoben2);
  i2clcd_Write_Rechts(I2CLCD_LINE2, lcdunten2);  
}

static void LCD_Show_Single(void)
{
  i2clcd_Write_Links(I2CLCD_LINE1, lcdoben);
  i2clcd_Write_Links(I2CLCD_LINE2, lcdunten);
  i2clcd_Write_Rechts(I2CLCD_LINE1, lcdoben);
  i2clcd_Write_Rechts(I2CLCD_LINE2, lcdunten);  
}

static void LCD_Augen_Blick(void)
{
  i2clcd_Write_Links(I2CLCD_LINE1, lcdtxt);
  i2clcd_Write_Links(I2CLCD_LINE2, lcdtxt);
  i2clcd_Write_Rechts(I2CLCD_LINE1, lcdtxt);
  i2clcd_Write_Rechts(I2CLCD_LINE2, lcdtxt);
}

static void LCD_Augen_Blinzeln(void)
{
  char leertxt[17] = "                ";

  i2clcd_Write_Links(I2CLCD_LINE1, leertxt);
  delay(BEWEGUNG);
  i2clcd_Write_Links(I2CLCD_LINE2, leertxt);
  i2clcd_Write_Rechts(I2CLCD_LINE1, leertxt);
  delay(BEWEGUNG);
  i2clcd_Write_Rechts(I2CLCD_LINE2, leertxt);
  delay(BEWEGUNG);
  i2clcd_Write_Links(I2CLCD_LINE2, lcdtxt);
  delay(BEWEGUNG);
  i2clcd_Write_Links(I2CLCD_LINE1, lcdtxt);
  i2clcd_Write_Rechts(I2CLCD_LINE2, lcdtxt);
  delay(BEWEGUNG);
  i2clcd_Write_Rechts(I2CLCD_LINE1, lcdtxt);
  delay(BEWEGUNG);
  delay(BEWEGUNG);
}
